require "optparse"
require "json"
require "uri"
require "ostruct"
require "aws-sdk-sqs"
require "webload/client/cli"
require "webload/client/logger"
require "webload/client/version"
require "webload/client/log_file"
require "webload/client/sqs"

module Webload
  module Client
    time = Time.now
    rate = $options.rpm.zero? ? 0 : 60.0 / $options.rpm

    fd = IO.popen("sudo tail -F -n0 #{$options.log_file}", 'r')
    $logger.debug('Log file opened and ready for processing entries')
    entries = []

    fd.each_line do |line|
      begin
        now = Time.now
        interval = now - time

        if match = line.match(/(GET|POST) (.*?) HTTP.* ("\{*.*\}*")/)
          method, uri, post_data = match.captures
        else
          next
        end

        next if method.nil?
        next if $options.uri_ignore && uri.match(/#{$options.uri_ignore}/)
        next if $options.methods_ignore && method.match(/#{$options.methods_ignore}/)

        # rate is 60 / requests per minute divided by 
        # interval is the difference between Time.now - time (Time.now at some time)
        # Rate 0 is send all messages
        # 60 RPM => rate = 1, so only after interval (time elapsed) is greater than rate
        # send the request and reset time. So only after the interval is greater than 1 second
        # 120 RPM => rate = 0.5, so only after 0.5 seconds passed, send the request.
        if interval > rate
          time = Time.now
          request = URI.join($options.url, uri)
          message = {
            method: method,
            url: request.to_s
          }

          if method == "POST"
            message[:post_data] = post_data.undump
          end

          if $options.basic_auth
            message[:basic_auth] = $options.basic_auth
          end

          message = message.to_json
          $logger.debug('New message: ' + message)
          entries << {id: entries.size.to_s, message_body: message}
  
          if entries.size == $options.batch
            $options.multiply_by.times do 
              $client.send_message_batch(queue_url: $queue, entries: entries)
              $logger.debug('Batch messages sent to SQS')
            end
            entries = []
          end
        end
      rescue => e
        $logger.warn(e.message)
      end
    end
  end
end
